#!/usr/bin/perl -w

# Copyright 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Graph::Maker::TwinAlternateAreaTree;


my $max_level = 6;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = 'gen';

my %arrays;
foreach my $k (0 .. $max_level) {
  my $graph = Graph::Maker->new('twin_alternate_area_tree', level=>$k,
                                undirected=>1);
  my $Sv = 0;
  my $Cv = 1 << ($k-1);

  my %degree1 = map {$graph->vertex_degree($_)==1
                       ? ($_ => 1) : ()} $graph->vertices;

  my $TAW = 0;
  my $TAwS = 0;
  my $TAwC = 0;
  my $TAW_count = 0;
  my $TAW_count_distinct = 0;

  my $TATW = 0;
  my $TAtwS = 0;
  my $TAtwC = 0;
  my $TATW_count = 0;
  my $TATW_count_distinct = 0;

  my $for = $graph->for_shortest_paths
    (sub {
       my ($t, $u,$v, $n) = @_;
       my $path_length = $t->path_length($u,$v);
       if ($u <= $v) {   # one direction only
         $TAW += $path_length;
         $TAW_count++;
         if ($u < $v) { $TAW_count_distinct++; }

         if ($degree1{$u} && $degree1{$v}) {
           $TATW += $path_length;
           $TATW_count++;
           if ($u < $v) { $TATW_count_distinct++; }
         }
       }
       if ($u eq $Sv) {
         $TAwS += $path_length;
         if ($degree1{$v}) { $TAtwS += $path_length; }
       }
       if ($u eq $Cv) {
         $TAwC += $path_length;
         if ($degree1{$v}) { $TAtwC += $path_length; }
       }
     });
  push @{$arrays{'TAW'}},   $TAW;
  push @{$arrays{'TAwS'}},  $TAwS;
  push @{$arrays{'TAwC'}},  $TAwC;
  push @{$arrays{'TAW_count'}}, $TAW_count;
  push @{$arrays{'TAW_count_distinct'}}, $TAW_count_distinct;

  push @{$arrays{'TATW'}},   $TATW;
  push @{$arrays{'TAtwS'}},  $TAtwS;
  push @{$arrays{'TAtwC'}},  $TAtwC;
  push @{$arrays{'TATW_count'}}, $TATW_count;
  push @{$arrays{'TATW_count_distinct'}}, $TATW_count_distinct;
}

foreach my $name ('TAW',  'TAW_count',  'TAW_count_distinct',  'TAwS', 'TAwC',
                  'TATW', 'TATW_count', 'TATW_count_distinct', 'TAtwS','TAtwC',
                 ) {
  my $aref = $arrays{$name};
  print "% GP-DEFINE  ${var}_${name}_samples = [", join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
